import logging
import baco

logger = logging.getLogger('ipspeaker')

class RtpStreamer:
    def __init__(self,pcmcDev,url,audioProfile) -> None:
        """
        IPAC RTP Streamer
        :param pcmcDev: pcm capture device
        :param url: streamer url
        :param audioProfile: audio format profile
        """
        self.pcmcDev = pcmcDev
        self.url = url
        self.audioProfile = audioProfile
        logger.info(f"RTP Streamer audio profile: {self.audioProfile}")
        self.codecId = None
        self.sampleRate = None
        self.channelId = None
        self.streamer = None

    def loadProfileParams(self):
        if self.audioProfile == 'PCM16BE':
            self.codecId = baco.CodecId.S16BE
            self.sampleRate = 44100
            self.channelId = baco.ChannelId.all_channels
        elif self.audioProfile == 'MP3_128K_44100_STEREO':
            self.codecId = baco.CodecId.MP3
            self.sampleRate = 44100
            self.channelId = baco.ChannelId.all_channels
        elif self.audioProfile == 'G722':
            self.codecId = baco.CodecId.G722
            self.sampleRate = 16000
            self.channelId = baco.ChannelId.front_left
        elif self.audioProfile == 'G711_ULAW':
            self.codecId = baco.CodecId.MULAW
            self.sampleRate = 8000
            self.channelId = baco.ChannelId.front_left
        elif self.audioProfile == 'G711_ALAW':
            self.codecId = baco.CodecId.ALAW
            self.sampleRate = 8000
            self.channelId = baco.ChannelId.front_left
        else:
            logger.error(f"Invalid audio profile: {self.audioProfile}")
            return False
        return True

    def startStreamer(self):
        self.streamer = baco.GenericStreamer(self.pcmcDev, 44100)
        endpoint = baco.StreamerEndpoint(self.url, self.codecId, self.sampleRate)
        self.streamer.addEndpoint(endpoint, self.channelId)
        endpoint.enable()
        logger.info(f"Start streaming to {self.url}, codec: {self.codecId}, sample rate: {self.sampleRate}, channelId: {self.channelId}")
        self.streamer.start()

    def stopStreamer(self):
        if self.streamer is not None:
            logger.info("Stop streaming...")
            self.streamer.stop()